<?php

namespace Drupal\image_sizes\Plugin\Field\FieldFormatter;

use Drupal\file\Entity\File;
use Drupal\Component\Utility\Html;
use Drupal\image\Entity\ImageStyle;
use Drupal\Core\Field\FormatterBase;
use Drupal\Component\Serialization\Json;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\image_sizes\Entity\ImageSizesPresetEntity;


/**
 * Plugin implementation of the 'image_sizes_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "image_sizes_preset_formatter",
 *   label = @Translation("Image sizes presets"),
 *   field_types = {
 *     "image"
 *   }
 * )
 */
class ImageSizesPresetFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      // Implement default settings.
      'preset' => false,
    ] + parent::defaultSettings();
  }

  function getPreset() {
    return \Drupal::entityTypeManager()
      ->getStorage('image_sizes_preset_entity')
      ->load($this->getSetting('preset'));
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    return [
      // Implement settings form.
      'preset' => [
        '#type' => 'select',
        '#required' => true,
        '#title' => $this->t('Placeholder image style.'),
        '#options' => $this->getOptions(),
        '#default_value' => $this->getSetting('preset')
      ],
    ] + parent::settingsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    // Implement settings summary.
    $preset = $this->getPreset();
    if ($preset) {
      $summary['#markup'] = $preset->label();
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    foreach ($items as $delta => $item) {
      $elements[$delta] = $this->viewValue($item);
    }
    $elements['#attached']['library'] = ['image_sizes/core'];

    return $elements;
  }

  protected function getOptions() {
    $presets = \Drupal::entityQuery('image_sizes_preset_entity')
      ->execute();
    $options = [];
    foreach( $presets as $preset ) {
      if ($preset = ImageSizesPresetEntity::load($preset)) {
        $options[$preset->id()] = $preset->label();
      }
    }
    return $options;
  }

  protected function getStyles($uri) {
    $preset = $this->getPreset();
    $image_styles = ImageStyle::loadMultiple($preset->getStyles());
    foreach( $image_styles as $style ) {
      $effects = $style->getEffects();
      foreach ($effects as $effect) {
        if (is_a($effect, 'Drupal\image\Plugin\ImageEffect\ResizeImageEffect')) {
          $width = $effect->getConfiguration()['data']['width'];
          if ($width !== null && $width > 0) {
            $styles[$width] = $style->buildUrl($uri);
          }
        }
      }
    }
    ksort($styles);
    return $styles;
  }

  protected function getFallBackStyle($uri) {
    $preset = $this->getPreset();

    if ($preset->getFallbackStyle() == 'original') {
      return file_create_url($uri);
    }

    $style = ImageStyle::load($preset->getFallbackStyle());
    if (!$style) {
      return file_create_url($uri);
    }

    return $style->buildUrl($uri);
  }

  protected function getPreloadStyle($uri) {
    $preset = $this->getPreset();

    if ($preset->getPreloadStyle() == 'original') {
      return file_create_url($uri);
    }

    $style = ImageStyle::load($preset->getPreloadStyle());
    if (!$style) {
      return file_create_url($uri);
    }

    return $style->buildUrl($uri);
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $dependencies = parent::calculateDependencies();
    $preset = $this->getPreset();

    $dependencies[$preset->getConfigDependencyKey()][] = $preset->getConfigDependencyName();

    return $dependencies;
  }

  /**
   * Generate the output appropriate for one field item.
   *
   * @param \Drupal\Core\Field\FieldItemInterface $item
   *   One field item.
   *
   * @return string
   *   The textual output generated.
   */
  protected function viewValue(FieldItemInterface $item) {
    $file = File::load($item->target_id);
    if (!$file) {
      return [];
    }
    $preset = $this->getPreset();
    $render = [
      '#theme' => 'image_style',
      '#style_name' => $preset->getPreloadStyle(),
      '#uri' => $file->getFileUri(),
      '#attributes' => [
        'data-src' => Json::encode($this->getStyles($file->getFileUri())),
        'class' => [
          'image-sizes',
          'pre-load'
        ],
        'data-src-fallback' => $this->getFallBackStyle($file->getFileUri()),
      ],
      // optional parameters
    ];
    return $render;
  }

}
